//
//  RedirectResolver.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Webteh.hr. All rights reserved.
//

#import "RedirectResolver.h"

const int STARTED_TRANSACTION = 0;
const int TRANSACTION_SUCCESS = 1;
const int TRANSACTION_FAILURE = 2;
const int TIMEOUT = 3;
const int UNKNOWN = 4;
const int THREE_DS = 5;
const int STARTED_TRANSACTION_3DS = 6;
const int IGNORE = 2121;


@interface RedirectResolver()


@property bool transactionInProgress;
@property long timePassed;
@property long timeout;
@property long timeoutHandlerPeriod;
@property(strong, nonatomic) NSTimer* timer;

-(void) stopTimeoutTracking;
-(void) startTimeoutTracking;

@end

@implementation RedirectResolver

-(id) init: (TransactionLifecycleAdapter *) adapter
   timeout: (long) timeout
{
    self.transactionLifecycleAdapter = adapter;
    self.transactionInProgress = false;
    _timePassed = 0;
    _timeout = timeout;
    _timeoutHandlerPeriod = 500;
    return [self init];
}

-(int) resolve:(NSString *)request{
    if ([request rangeOfString:@"/api/add-card"].location != NSNotFound || [request rangeOfString:@"/api/pay-cvv"].location != NSNotFound) {
        return STARTED_TRANSACTION;
    } else if([request rangeOfString:@"/api/pares"].location != NSNotFound){
        return STARTED_TRANSACTION_3DS;
    } else if([request rangeOfString:@"/api/success"].location != NSNotFound) {
        return TRANSACTION_SUCCESS;
    } else if([request rangeOfString:@"/api/error"].location != NSNotFound){
        return TRANSACTION_FAILURE;
    } else if([request rangeOfString:@"/default/add-card"].location != NSNotFound || [request rangeOfString:@"/default/pay-cvv"].location != NSNotFound){
        return IGNORE;
    } else if([request rangeOfString:@"/default/3ds"].location != NSNotFound){
        return THREE_DS;
    }
    else{
        return UNKNOWN;
    }
}
    
-(void) logTransactionInfo:(NSString *)loggingInfo {
    if(_transactionLifecycleAdapter.delegate && [_transactionLifecycleAdapter.delegate respondsToSelector:@selector(logTransactionInfo:)]){
        [_transactionLifecycleAdapter.delegate logTransactionInfo:loggingInfo];
    }
}

-(void) notifyDelegate:(int)resolvedCode request:(NSString *)request {
    [self logTransactionInfo:[NSString stringWithFormat:@"notifyDelegate: resolvedCode: %d", resolvedCode]];
    
    NSArray *testArray = request == nil ? nil: [request componentsSeparatedByString:@"/"];
    NSString *inputParam = (testArray == nil || testArray.count == 0)? nil: testArray[testArray.count - 1];
    
    switch (resolvedCode) {
        case STARTED_TRANSACTION:
        case STARTED_TRANSACTION_3DS:
            _transactionInProgress = true;
            [self startTimeoutTracking];
            [_transactionLifecycleAdapter.delegate showProgress];
            
            if (resolvedCode == STARTED_TRANSACTION_3DS){
                if(_transactionLifecycleAdapter.delegate && [_transactionLifecycleAdapter.delegate respondsToSelector:@selector(threeDsSubmitted)]){
                    [self logTransactionInfo:[NSString stringWithFormat:@"_transactionLifecycleAdapter.delegate#threeDsSubmitted"]];
                    [_transactionLifecycleAdapter.delegate threeDsSubmitted];
                }
            }
            
            break;
        case TRANSACTION_SUCCESS:{
            // If transaction is not in progress there is no need for transaction success handling
            // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
            // After a while transaction success or transaction failure occurs.
            if(!_transactionInProgress){
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction success invoked, but transactionInProgress = false. Ignoring."]];
                return;
            }
            
            _transactionInProgress = false;
            
            [self stopTimeoutTracking];
            
            [_transactionLifecycleAdapter.delegate hideProgress];
            
            TransactionSuccess *success = [TransactionSuccess fromString:inputParam];
            
            [self logTransactionInfo:[NSString stringWithFormat:@"Transaction success %@", success]];
            
            [_transactionLifecycleAdapter.delegate transactionSuccess:success];
            break;
        }
        case TRANSACTION_FAILURE:{
            // If transaction is not in progress there is no need for transaction success handling
            // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
            // After a while transaction success or transaction failure occurs.
            if(!_transactionInProgress){
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction success invoked, but transactionInProgress = false. Ignoring."]];
                return;
            }
            
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter.delegate hideProgress];
            NSDictionary* decodedInput = [InputResolver decode:inputParam];
            BusinessError* businessError = [BusinessError fromJson:decodedInput];
            
            if (businessError) {
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction failure %@", businessError]];
                [_transactionLifecycleAdapter transactionFailure:businessError];
            } else{
                ApiError *apiError = [ApiError fromJson:decodedInput];
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction failure %@", apiError]];
                [_transactionLifecycleAdapter transactionFailure:apiError];
            }
            
            break;
        }
        case THREE_DS:
            // If transaction is not in progress there is no need for transaction success handling
            // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
            // After a while transaction success or transaction failure occurs.
            if(!_transactionInProgress){
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction 3ds invoked, but transactionInProgress = false. Ignoring."]];
                return;
            }
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter.delegate hideProgress];
            
            if(_transactionLifecycleAdapter.delegate && [_transactionLifecycleAdapter.delegate respondsToSelector:@selector(threeDsLoaded)]){
                [self logTransactionInfo:[NSString stringWithFormat:@"_transactionLifecycleAdapter.delegate#threeDsLoaded"]];
                [_transactionLifecycleAdapter.delegate threeDsLoaded];
            }
            break;
            
        case TIMEOUT:
            // If transaction is not in progress there is no need for transaction success handling
            // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
            // After a while transaction success or transaction failure occurs.
            if(!_transactionInProgress){
                [self logTransactionInfo:[NSString stringWithFormat:@"Transaction timeout invoked, but transactionInProgress = false. Ignoring."]];
                return;
            }
            
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            
            [_transactionLifecycleAdapter timeout];
            break;
        case UNKNOWN:
            break;
        default:
            [self logTransactionInfo:[NSString stringWithFormat:@"Unable to resolve %d", resolvedCode]];
            break;
    }
}

-(void) stopTimeoutTracking{
    if (_timer) {
        [self logTransactionInfo:@"Stoping timeout tracking"];
        [_timer invalidate];
        _timer = nil;
    }
}
-(void) startTimeoutTracking{
    
    [self logTransactionInfo:@"Starting timeout tracking"];
    
    _timer = [NSTimer scheduledTimerWithTimeInterval:1.0
                                              target:self
                                            selector:@selector(onTick:)
                                            userInfo:nil
                                             repeats:YES];
}

-(void)onTick:(NSTimer *)timer {
    _timePassed += _timeoutHandlerPeriod;
    if (_timePassed >= _timeout) {
        _timePassed = 0;
        [self logTransactionInfo:[NSString stringWithFormat:@"Transaction timeout with timeout: %d", _timeout]];
        [self notifyDelegate:TIMEOUT request:nil];
    }
}

-(void) dealloc{
    [_timer invalidate];
    _timer = nil;
}
@end
